import math
import pyfie

# 利便のため PyFIE 関数が返すエラーコードに応じて例外を発生させる機能を有効化
pyfie.ctrl.enable_f_err_exception(True)

# 極座標変換しないボーダー部分のサイズ
# 濃度補間時に画像範囲外となり画像が欠けないようにするために必要な定数です。
# 条件によってより大きな値にする必要があります。
BORDER_SIZE = 1

# 処理対象画像の読み込み
image = pyfie.imread("polar_src_lower.png", layered=True)

# 1. 順変換
# 出力画像生成。半径を入力画像の高さに合わせる
image_polar = pyfie.fnFIE_img_root_alloc(
    image.f_type, image.ch, image.height * 2, image.height * 2)
radius = image.height

# 入力画像の左下を基準に(下辺が回転軸)
coord_mode = pyfie.F_PT_LOWER
origin_in = pyfie.DPNT_T(BORDER_SIZE, image.height - 1 - BORDER_SIZE)
# 出力画像の画像中心が回転中心
origin_out = pyfie.DPNT_T(image_polar.height / 2, image_polar.height / 2)
# X軸負方向から時計回りに1周するように出力
start_angle = math.pi
range_angle = 2.0 * math.pi

# 順変換実行
pyfie.fnFIE_polar_trans_img(
    image,  # 入力画像
    image_polar,  # 出力画像
    origin_in,  # 入力側での処理対象領域の開始座標
    image.width - BORDER_SIZE * 2,  # 入力側での処理対象領域の幅
    image.height - BORDER_SIZE * 2,  # 入力側での処理対象領域の高さ
    origin_out,  # 出力側での円板の回転中心
    radius,  # 出力側での円板の半径
    start_angle,  # 出力側での処理開始角度
    range_angle,  # 出力側での角度の処理範囲
    coord_mode,  # 入力側での回転中心位置
    True,  # 出力画像の背景処理方法
    pyfie.F_SAMPLING_BILINEAR  # 濃度補間法
)

# 画像オブジェクトのプロット
image_polar.imshow()
# プロットした画像の表示
pyfie.plot_show()
# 極座標変換画像を保存
pyfie.imwrite("polar_dst_lower.png", image_polar)

# 2. 逆変換
# 画像確保。入力画像と同じ型とサイズ
image_polar_inv = image.empty_like()

# 逆変換では基準座標の入力・出力が逆になる
origin_in_inv, origin_out_inv = origin_out, origin_in

# 逆変換実行
pyfie.fnFIE_polar_trans_img_inv(
    image_polar,  # 入力画像
    image_polar_inv,  # 出力画像
    origin_in_inv,  # 入力側での処理対象領域の開始座標
    radius,  # 入力側での円板の半径
    start_angle,  # 入力側での処理開始角度
    range_angle,  # 入力側での角度の処理範囲
    origin_out_inv,  # 出力側での円板の回転中心
    image.width - BORDER_SIZE * 2,  # 出力側での処理対象領域の幅
    image.height - BORDER_SIZE * 2,  # 出力側での処理対象領域の高さ
    coord_mode,  # 出力側での回転中心位置
    True,  # 出力画像の背景処理方法
    pyfie.F_SAMPLING_BILINEAR  # 濃度補間法
)

# 結果出力
# 元の画像とほとんど等しい画像が得られる。
# 0クリアされた画素の影響などで、周辺部数画素は上手く復元されない。
# 画像オブジェクトのプロット
image_polar_inv.imshow()
# プロットした画像の表示
pyfie.plot_show()
# 極座標変換画像を保存
pyfie.imwrite("polar_dst_inv_lower.png", image_polar_inv)
